package com.nareason.share_content_facebook;


import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.net.Uri;
import android.util.Log;

import com.facebook.CallbackManager;
import com.facebook.FacebookCallback;
import com.facebook.FacebookException;
import com.facebook.share.Sharer;
import com.facebook.share.internal.ShareFeedContent;
import com.facebook.share.model.ShareContent;
import com.facebook.share.model.ShareHashtag;
import com.facebook.share.model.ShareLinkContent;
import com.facebook.share.model.ShareMediaContent;
import com.facebook.share.model.SharePhoto;
import com.facebook.share.model.SharePhotoContent;
import com.facebook.share.model.ShareVideo;
import com.facebook.share.widget.ShareDialog;

import java.io.ByteArrayOutputStream;

import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.PluginRegistry;

/** ShareContentFacebookPlugin */
public class ShareContentFacebookPlugin implements MethodChannel.MethodCallHandler {
  private final Context context;
  private final Activity activity;
  private final CallbackManager callbackManager;

  /**
   * Plugin registration.
   */
  public static void registerWith(PluginRegistry.Registrar registrar) {
    final MethodChannel channel = new MethodChannel(registrar.messenger(), "share_content_facebook");
    channel.setMethodCallHandler(new ShareContentFacebookPlugin(registrar));
  }

  private ShareContentFacebookPlugin(PluginRegistry.Registrar registrar) {
    this.context = registrar.context();
    this.activity = registrar.activity();
    callbackManager = CallbackManager.Factory.create();

    registrar.addActivityResultListener(new PluginRegistry.ActivityResultListener() {
      @Override
      public boolean onActivityResult(int requestCode, int resultCode, Intent data) {
        callbackManager.onActivityResult(requestCode, resultCode, data);
        return true;
      }
    });
  }

  @Override
  public void onMethodCall(MethodCall call, MethodChannel.Result result) {
    if (call.method.equals("shareContent")) {
      String url = call.argument("url");
      String quote = call.argument("quote");
      String hashTag = call.argument("hashTag");
      shareContent(result, url, quote,hashTag);
    }else if (call.method.equals("sharePhoto")) {
      byte[] pathImage = call.argument("pathImage");
      sharePhoto(result, pathImage);
    }  else {
      result.notImplemented();
    }
  }

  private void shareContent(final MethodChannel.Result result, String url, String quote,String hashTag) {
    ShareLinkContent.Builder builder = new ShareLinkContent.Builder();
    if (url != null && !url.isEmpty()){
      Uri uri = Uri.parse(url);
      if(uri != null){
        builder.setContentUrl(uri);
      }
    }

    if (quote != null && !quote.isEmpty()){
      builder.setQuote(quote);
    }

    if (hashTag != null && !hashTag.isEmpty()) {
      builder.setShareHashtag(new ShareHashtag.Builder().setHashtag(hashTag).build());
    }
    ShareDialog shareDialog = new ShareDialog(this.activity);
    shareDialog.registerCallback(callbackManager, new FacebookCallback<Sharer.Result>() {

      @Override
      public void onSuccess(Sharer.Result shareResult) {
        result.success(true);
      }

      @Override
      public void onCancel() {
        result.success(false);
      }

      @Override
      public void onError(FacebookException error) {
        result.error("FacebookException", error.getMessage(), null);
      }
    });
    shareDialog.show(builder.build(), ShareDialog.Mode.AUTOMATIC);
  }

  private void sharePhoto(final MethodChannel.Result result,byte[] image) {

    if(image != null){
      /*Bitmap bitmap = BitmapFactory.decodeFile(image);
      ByteArrayOutputStream blob = new ByteArrayOutputStream();
      bitmap.compress(Bitmap.CompressFormat.PNG, 0 /* Ignored for PNGs *///, blob);
      //byte[] bitmapdata = blob.toByteArray();
      Bitmap bitmap = BitmapFactory.decodeByteArray(image, 0, image.length);
      SharePhoto photo = new SharePhoto.Builder()
              .setBitmap(bitmap)
              .build();
      SharePhotoContent content = new SharePhotoContent.Builder()
              .addPhoto(photo)
              .build();

      ShareDialog shareDialog = new ShareDialog(this.activity);
      shareDialog.registerCallback(callbackManager, new FacebookCallback<Sharer.Result>() {

        @Override
        public void onSuccess(Sharer.Result shareResult) {
          result.success(true);
        }

        @Override
        public void onCancel() {
          result.success(false);
        }

        @Override
        public void onError(FacebookException error) {
          result.error("FacebookException", error.getMessage(), null);
        }
      });
      shareDialog.show(content, ShareDialog.Mode.AUTOMATIC);
    }

  }

}
